// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PlayerController.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

PlayerController::PlayerController(SceneItem *item) : SceneItemComponent(item) {

}

std::vector<SceneItemComponent*> PlayerController::getDependencies() {
  return {
    (this->characterController = item->getComponent<CharacterController2D>()),
    (this->entityHealth = item->getComponent<EntityHealth>())
  };
}

void PlayerController::onStart() {
  assertNotNull(this->characterController);
  assertNotNull(this->entityHealth);

  useEvent([&](float_t delta){
    // Don't move if stunned.
    if(entityHealth->isStunned()) return;

    // Movement
    auto inputMove = getGame()->inputManager.getAxis2D(
      INPUT_BIND_NEGATIVE_X, INPUT_BIND_POSITIVE_X,
      INPUT_BIND_NEGATIVE_Y, INPUT_BIND_POSITIVE_Y
    );
    
    if(inputMove.x != 0 || inputMove.y != 0) {
      float_t angle = atan2(inputMove.y, inputMove.x);
      glm::vec2 movement(cos(angle), sin(angle));

      transform->setLocalRotation(glm::quat(glm::vec3(0, -angle + 1.5708f, 0)));

      characterController->velocity += movement * delta * moveSpeed;
    }
  }, getScene()->eventSceneUpdate);

  // Hurt Hazard Processing
  useEvent([&](struct CharacterController2DCollisionEventInfo info) {
    if(entityHealth->isInvincible()) return;

    auto hurtHazard = info.collider->item->getComponent<HurtHazard>();
    if(hurtHazard == nullptr) return;

    glm::vec2 back = glm::vec2(
      info.collider->transform->getLocalPosition().x,
      info.collider->transform->getLocalPosition().z
    ) - glm::vec2(
      this->transform->getLocalPosition().x,
      this->transform->getLocalPosition().z
    );

    this->characterController->velocity = back * -hitKnockback;
    this->entityHealth->damage({
      .amount = hurtHazard->damage
    });
  }, this->characterController->eventCollision);

}